"use strict";

var _interopRequireDefault = require("@babel/runtime-corejs3/helpers/interopRequireDefault");

var _Object$defineProperty = require("@babel/runtime-corejs3/core-js-stable/object/define-property");

_Object$defineProperty(exports, "__esModule", {
  value: true
});

exports.Logger = void 0;

var _indexOf = _interopRequireDefault(require("@babel/runtime-corejs3/core-js-stable/instance/index-of"));

var _concat = _interopRequireDefault(require("@babel/runtime-corejs3/core-js-stable/instance/concat"));

var _trim = _interopRequireDefault(require("@babel/runtime-corejs3/core-js-stable/instance/trim"));

var _stringify = _interopRequireDefault(require("@babel/runtime-corejs3/core-js-stable/json/stringify"));

var _slice = _interopRequireDefault(require("@babel/runtime-corejs3/core-js-stable/instance/slice"));

var _filter = _interopRequireDefault(require("@babel/runtime-corejs3/core-js-stable/instance/filter"));

var _map = _interopRequireDefault(require("@babel/runtime-corejs3/core-js-stable/instance/map"));

var _splice = _interopRequireDefault(require("@babel/runtime-corejs3/core-js-stable/instance/splice"));

var _from = _interopRequireDefault(require("@babel/runtime-corejs3/core-js-stable/array/from"));

var _keys = _interopRequireDefault(require("@babel/runtime-corejs3/core-js-stable/object/keys"));

require('colors');

const fs = require('fs');

const date = require('date-and-time');

const {
  escapeHtml
} = require('embark-utils');

const util = require('util');

const DATE_FORMAT = 'YYYY-MM-DD HH:mm:ss:SSS';
const LOG_REGEX = new RegExp(/\[(\d\d\d\d-\d\d-\d\d \d\d:\d\d:\d\d:\d\d\d)\] (?:\[(\w*)\]:?)?\s?\s?(.*)/gmi);

class Logger {
  constructor(options) {
    this.events = options.events || {
      emit: function () {}
    };
    this.logLevels = (0, _keys.default)(Logger.logLevels);
    this.logLevel = options.logLevel || 'info';
    this._logFunction = options.logFunction || console.log;

    this.logFunction = function () {
      var _context;

      const args = (0, _from.default)(arguments);
      const color = args[args.length - 1];
      (0, _splice.default)(args).call(args, args.length - 1, 1);

      this._logFunction(...(0, _map.default)(_context = (0, _filter.default)(args).call(args, arg => arg !== undefined && arg !== null)).call(_context, arg => {
        if (color) {
          return typeof arg === 'object' ? util.inspect(arg, 2)[color] : arg[color];
        }

        return typeof arg === 'object' ? util.inspect(arg, 2) : arg;
      }));
    };

    this.logFile = options.logFile;
    this.context = options.context;
  }
  /**
   * Parses the logFile, returning an array of JSON objects containing the
   * log messages.
   * @param {Number} limit specifies how many log messages to return from the
   *  end of the log file
   * @returns {Array} array containing
   *  - msg: the log message
   *  - logLevel: log level (ie 'info', 'debug')
   *  - name: process name (always "embark")
   *  - timestamp: timestamp of log message (milliseconds since 1/1/1970)
   */


  parseLogFile(limit) {
    let matches;
    let logs = [];
    const logFile = fs.readFileSync(this.logFile, 'utf8');

    while ((matches = LOG_REGEX.exec(logFile)) !== null) {
      // This is necessary to avoid infinite loops with zero-width matches
      if (matches.index === LOG_REGEX.lastIndex) {
        LOG_REGEX.lastIndex++;
      }

      if (matches && matches.length) {
        logs.push({
          msg: [matches[3]],
          logLevel: matches[2],
          name: 'embark',
          timestamp: date.parse(matches[1], DATE_FORMAT).getTime()
        });
      }
    } // if 'limit' is specified, get log lines from the end of the log file


    if (limit && limit > 0 && logs.length > limit) {
      (0, _slice.default)(logs).call(logs, limit * -1);
    }

    return logs;
  }

}

exports.Logger = Logger;
Logger.logLevels = {
  error: 'error',
  warn: 'warn',
  info: 'info',
  debug: 'debug',
  trace: 'trace'
};

Logger.prototype.registerAPICall = function (plugins) {
  const self = this;
  let plugin = plugins.createPlugin('dashboard', {});
  plugin.registerAPICall('ws', '/embark-api/logs', (ws, _req) => {
    self.events.on("log", function (logLevel, logMsg) {
      logMsg = escapeHtml(logMsg);
      ws.send((0, _stringify.default)({
        msg: logMsg,
        msg_clear: logMsg.stripColors,
        logLevel: logLevel
      }), () => {});
    });
  });
};

Logger.prototype.writeToFile = function (_txt) {
  var _context2;

  if (!this.logFile) {
    return;
  }

  let origin = "[" + (0, _trim.default)(_context2 = new Error().stack.split("at ")[3]).call(_context2) + "]";
  const formattedDate = [`[${date.format(new Date(), DATE_FORMAT)}]`]; // adds a timestamp to the logs in the logFile

  fs.appendFileSync(this.logFile, "\n" + (0, _concat.default)(formattedDate).call(formattedDate, origin, (0, _from.default)(arguments)).join(' '));
};

Logger.prototype.error = function () {
  if (!arguments.length || !this.shouldLog('error')) {
    return;
  }

  this.events.emit("log", "error", ...arguments);
  this.logFunction(...(0, _from.default)(arguments), 'red');
  this.writeToFile("[error]: ", ...arguments);
};

Logger.prototype.warn = function () {
  if (!arguments.length || !this.shouldLog('warn')) {
    return;
  }

  this.events.emit("log", "warn", ...arguments);
  this.logFunction(...(0, _from.default)(arguments), 'yellow');
  this.writeToFile("[warning]: ", ...arguments);
};

Logger.prototype.info = function () {
  if (!arguments.length || !this.shouldLog('info')) {
    return;
  }

  this.events.emit("log", "info", ...arguments);
  this.logFunction(...(0, _from.default)(arguments), 'green');
  this.writeToFile("[info]: ", ...arguments);
};

Logger.prototype.consoleOnly = function () {
  if (!arguments.length || !this.shouldLog('info')) {
    return;
  }

  this.logFunction(...(0, _from.default)(arguments), 'green');
  this.writeToFile("[consoleOnly]: ", ...arguments);
};

Logger.prototype.debug = function () {
  if (!arguments.length || !this.shouldLog('debug')) {
    return;
  }

  this.events.emit("log", "debug", ...arguments);
  this.logFunction(...arguments, null);
  this.writeToFile("[debug]: ", ...arguments);
};

Logger.prototype.trace = function () {
  if (!arguments.length || !this.shouldLog('trace')) {
    return;
  }

  this.events.emit("log", "trace", ...arguments);
  this.logFunction(...arguments, null);
  this.writeToFile("[trace]: ", ...arguments);
};

Logger.prototype.dir = function (txt) {
  if (!txt || !this.shouldLog('info')) {
    return;
  }

  this.events.emit("log", "dir", txt);
  this.logFunction(txt, null);
  this.writeToFile("[dir]: ", ...arguments);
};

Logger.prototype.shouldLog = function (level) {
  var _context3, _context4;

  return (0, _indexOf.default)(_context3 = this.logLevels).call(_context3, level) <= (0, _indexOf.default)(_context4 = this.logLevels).call(_context4, this.logLevel);
};
//# sourceMappingURL=index.js.map